unit janArrowButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, janButtonUtils;

type
  TjanArrowButton = class(TButton)
  private
    bm:TBitmap;
    IsFocused: Boolean;
    IsHot:boolean;
    FCanvas: TCanvas;
    FHotColor: TColor;
    FFlat: boolean;
    FFlatBorderColor: TColor;
    FFlatArrow: boolean;
    procedure CMMouseLeave(var Message: TMessage); message CM_MouseLeave;
    procedure CMMouseEnter(var Message: TMessage); message CM_MouseEnter;
    procedure CNDrawItem(var Msg: TWMDrawItem); message CN_DRAWITEM;
    procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
    procedure CMEnabledChanged(var Msg: TMessage); message CM_ENABLEDCHANGED;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk);
      message WM_LBUTTONDBLCLK;
    procedure SetHotColor(const Value: TColor);
    procedure SetFlat(const Value: boolean);
    procedure SetFlatBorderColor(const Value: TColor);
    procedure SetFlatArrow(const Value: boolean);
  protected
    procedure SetBounds (ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure SetButtonStyle(ADefault: Boolean); override;
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Color;
    property HotColor:TColor read FHotColor write SetHotColor;
    property Flat:boolean read FFlat write SetFlat;
    property FlatBorderColor:TColor read FFlatBorderColor write SetFlatBorderColor;
    property FlatArrow:boolean read FFlatArrow write SetFlatArrow;
    property Width default 100;
    property Height default 50;
    property ParentShowHint;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnEnter;
    property OnExit;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('janButtons', [TjanArrowButton]);
end;

constructor TjanArrowButton.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  bm:=Tbitmap.create;
  SetBounds (Left, Top, 100, 50);
  FCanvas := TCanvas.Create;
  FHotcolor:=clblue;
  FFlatborderColor:=clwhite;
end;

destructor TjanArrowButton.Destroy;
begin
  inherited Destroy;
  bm.free;
  FCanvas.Free;
end;

procedure TjanArrowButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params
    do Style := Style or bs_OwnerDraw;
end;

procedure TjanArrowButton.CreateWnd;
var
  hRegion: THandle;
  poly: array[0..5] of Tpoint;
  x8,y2:integer;
begin
  inherited CreateWnd;
  if FFlatArrow then
    x8:=width div 16
  else
    x8:=width div 8;
  y2:=height div 2;
  poly[0]:=point(0,0);
  poly[1]:=point(width-x8,0);
  poly[2]:=point(width,y2);
  poly[3]:=point(width-x8,height);
  poly[4]:=point(0,height);
  poly[5]:=point(x8,y2);
  hRegion := CreatePolygonRgn(poly,6,WINDING);
  SetWindowRgn (Handle, hRegion, True);
end;

procedure TjanArrowButton.SetBounds (ALeft, ATop,
  AWidth, AHeight: Integer);
var
  hRegion: THandle;
  poly: array[0..5] of Tpoint;
  x8,y2:integer;
begin
  inherited SetBounds (ALeft, ATop, AWidth, AHeight);
  if HandleAllocated then
  begin
  if FFlatArrow then
    x8:=width div 16
  else
    x8:=width div 8;
  y2:=Aheight div 2;
  poly[0]:=point(0,0);
  poly[1]:=point(Awidth-x8,0);
  poly[2]:=point(Awidth,y2);
  poly[3]:=point(Awidth-x8,Aheight);
  poly[4]:=point(0,Aheight);
  poly[5]:=point(x8,y2);
  hRegion := CreatePolygonRgn(poly,6,WINDING);
  SetWindowRgn (Handle, hRegion, True);
  end;
end;

procedure TjanArrowButton.CNDrawItem(var Msg: TWMDrawItem);
var
  OdsDown, OdsFocus, ActionFocus: Boolean;
  Rect: TRect;
  poly: array[0..6] of Tpoint;
  polyBR:array[0..3] of TPoint;
  polyTL:array[0..3] of TPoint;
  x8,y2,w,h:integer;

  procedure setpoly;
  begin
    w:=Rect.right-Rect.left+1;
    h:=rect.bottom-Rect.top+1;
    if FFlatArrow then
      x8:=w div 16
    else
      x8:=w div 8;
    y2:=h div 2;
    poly[0]:=point(Rect.left,Rect.top);
    poly[1]:=point(rect.right-x8,Rect.top);
    poly[2]:=point(Rect.right,y2);
    poly[3]:=point(rect.right-x8,Rect.bottom);
    poly[4]:=point(0,rect.bottom);
    poly[5]:=point(x8,y2);
    poly[6]:=poly[0];
  end;

begin
  // initialize
  FCanvas.Handle := Msg.DrawItemStruct^.hDC;
  Rect := ClientRect;
  Dec (Rect.Right);
  Dec (Rect.Bottom);
  setpoly;
  with Msg.DrawItemStruct^ do
  begin
    OdsDown := itemState and ODS_SELECTED <> 0;
    OdsFocus := itemState and ODS_FOCUS <> 0;
    ActionFocus := ItemAction = oda_Focus
  end;
  bm.width:=width;
  bm.height:=height;
  with bm.Canvas do
  begin
    pen.width:=2;
    Brush.Color := Color;
    if not ActionFocus then
    begin
      // fill with current color
      Brush.Style := bsSolid;
      FillRect (Rect);
    end;
    // do not fill any more
    Brush.Style := bsClear;
    // draw border if default


{    if Default or OdsFocus then
    begin
      Pen.Color := clWindowFrame;
      if not ActionFocus then
        polyLine(poly);
      // reduce the area for further operations
      InflateRect (Rect, -1, -1);
    end;}
    // test code:
    //InflateRect (Rect, -1, -1);

    if FFlat and (not OdsDown) and (not IsHot) and (not (csdesigning in componentstate)) then
    begin
      Pen.Color := FFlatbordercolor;
      polyLine(poly);
    end
    else if OdsDown then
    begin
      // draw gray border all around
      Pen.Color := clBtnShadow;
      if not ActionFocus then
        polyline(poly);
      // gray border (bottom-right)
      Pen.Color :=  clwhite;
      setpoly;
      polyBR[0]:=poly[1];
      polyBR[1]:=poly[2];
      polyBR[2]:=poly[3];
      polyBR[3]:=poly[4];
      polyline(polyBR);
      // white border (top-left)
      Pen.Color :=  clwindowframe;
      polyTL[0]:=poly[4];
      polyTL[1]:=poly[5];
      polyTL[2]:=poly[0];
      polyTL[3]:=poly[1];
      polyline(polyTL);
      // gray border (bottom-right, internal)
      Pen.Color := clBtnShadow;
      InflateRect (Rect, -1, -1);
      setpoly;
      polyBR[0]:=poly[1];
      polyBR[1]:=poly[2];
      polyBR[2]:=poly[3];
      polyBR[3]:=poly[4];
      polyline(polyBR);
    end
    else if not ActionFocus then
    begin
      // gray border (bottom-right)
      Pen.Color :=  clWindowFrame;
      setpoly;
      polyBR[0]:=poly[1];
      polyBR[1]:=poly[2];
      polyBR[2]:=poly[3];
      polyBR[3]:=poly[4];
      polyline(polyBR);
      // white border (top-left)
      Pen.Color :=  clWhite;
      polyTL[0]:=poly[4];
      polyTL[1]:=poly[5];
      polyTL[2]:=poly[0];
      polyTL[3]:=poly[1];
      polyline(polyTL);
      // gray border (bottom-right, internal)
      Pen.Color := clBtnShadow;
      InflateRect (Rect, -1, -1);
      setpoly;
      polyBR[0]:=poly[1];
      polyBR[1]:=poly[2];
      polyBR[2]:=poly[3];
      polyBR[3]:=poly[4];
      polyline(polyBR);
    end;
    // smooth edges
    antialias(bm);
    // draw the caption
    InflateRect (Rect, - Width div 5, - Height div 5);
    if OdsDown then
    begin
      Inc (Rect.Left, 2);
      Inc (Rect.Top, 2);
    end;
    Font := Self.Font;
    if (IsHot and (not OdsDown)) then
      font.color:=FHotColor;
    if not ActionFocus then
      DrawText (bm.Canvas.Handle, PChar (Caption), -1,
        Rect, dt_SingleLine or dt_Center or dt_VCenter);

    // draw the focus rect around the text
    Brush.Style := bsSolid;
    Pen.Color:= clBlack;
    Brush.Color := clWhite;
    if IsFocused or OdsFocus or ActionFocus then
      DrawFocusRect (Rect);
  end; // with bm.Canvas and if DrawEntire
  Fcanvas.Draw (0,0,bm);
  FCanvas.Handle := 0;
  Msg.Result := 1; // message handled
end;

procedure TjanArrowButton.CMFontChanged(var Msg: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TjanArrowButton.CMEnabledChanged(var Msg: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TjanArrowButton.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  Perform(WM_LBUTTONDOWN, Message.Keys, Longint(Message.Pos));
end;

procedure TjanArrowButton.SetButtonStyle (ADefault: Boolean);
begin
  if ADefault <> IsFocused then
  begin
    IsFocused := ADefault;
    Invalidate;
  end;
end;


procedure TjanArrowButton.CMMouseEnter(var Message: TMessage);
begin
  IsHot:=true;
  invalidate;
end;

procedure TjanArrowButton.CMMouseLeave(var Message: TMessage);
begin
  IsHot:=false;
  invalidate;
end;

procedure TjanArrowButton.SetHotColor(const Value: TColor);
begin
  FHotColor := Value;
end;

procedure TjanArrowButton.SetFlat(const Value: boolean);
begin
  FFlat := Value;
  invalidate;
end;

procedure TjanArrowButton.SetFlatBorderColor(const Value: TColor);
begin
  FFlatBorderColor := Value;
end;

procedure TjanArrowButton.SetFlatArrow(const Value: boolean);
begin
  if value<>FFlatArrow then
  begin
    FFlatArrow := Value;
    setbounds(Left,top,width,height);
    invalidate;
  end;
end;

end.
