unit janRPentagonButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, janButtonUtils;

type
  TjanRPentagonButton = class(TButton)
  private
    bm:TBitmap;
    IsFocused: Boolean;
    IsHot:boolean;
    FCanvas: TCanvas;
    FHotColor: TColor;
    FFlat: boolean;
    FFlatBorderColor: TColor;
    FFlatArrow: boolean;
    procedure CMMouseLeave(var Message: TMessage); message CM_MouseLeave;
    procedure CMMouseEnter(var Message: TMessage); message CM_MouseEnter;
    procedure CNDrawItem(var Msg: TWMDrawItem); message CN_DRAWITEM;
    procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
    procedure CMEnabledChanged(var Msg: TMessage); message CM_ENABLEDCHANGED;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk);
      message WM_LBUTTONDBLCLK;
    procedure SetHotColor(const Value: TColor);
    procedure SetFlat(const Value: boolean);
    procedure SetFlatBorderColor(const Value: TColor);
    procedure calcpentagon(AWidth, Aheight:integer);
  protected
    procedure SetBounds (ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure SetButtonStyle(ADefault: Boolean); override;
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Color;
    property HotColor:TColor read FHotColor write SetHotColor;
    property Flat:boolean read FFlat write SetFlat;
    property FlatBorderColor:TColor read FFlatBorderColor write SetFlatBorderColor;
    property Width;
    property Height;
    property ParentShowHint;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnEnter;
    property OnExit;
  end;

procedure Register;

implementation

var xp,yp:integer;

procedure Register;
begin
  RegisterComponents('janButtons', [TjanRPentagonButton]);
end;

constructor TjanRPentagonButton.Create (AOwner: TComponent);
var r,x2,dy:integer;
    a:extended;

begin
  inherited Create (AOwner);
  bm:=Tbitmap.create;
  r:=50;
  a:=2*pi/5;
  x2:=round(r*cos(pi/2-a));
  dy:=round(r*cos(pi-2*a));
  SetBounds (Left, Top, 2*x2, r+dy);
  FCanvas := TCanvas.Create;
  FHotcolor:=clblue;
  FFlatborderColor:=clwhite;
end;

destructor TjanRPentagonButton.Destroy;
begin
  inherited Destroy;
  bm.free;
  FCanvas.Free;
end;

procedure TjanRPentagonButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params
    do Style := Style or bs_OwnerDraw;
end;

procedure TjanRPentagonButton.CreateWnd;
var
  hRegion: THandle;
  poly: array[0..4] of Tpoint;
  x2,y2:integer;
begin
  inherited CreateWnd;
  x2:=width div 2;
  y2:=height div 2;
  calcpentagon(width,height);
  poly[0]:=point(xp,0);
  poly[1]:=point(width-xp,0);
  poly[2]:=point(width,height-yp);
  poly[3]:=point(x2,height);
  poly[4]:=point(0,height-yp);
  hRegion := CreatePolygonRgn(poly,5,WINDING);
  SetWindowRgn (Handle, hRegion, True);
end;

procedure TjanRPentagonButton.calcpentagon(AWidth,Aheight:integer);
var
  x2,y2,r:integer;
  a:extended;
begin
  a:=pi/2-(2*pi/5);
  x2:=Awidth div 2;
  y2:=Aheight div 2;
  r:=round(x2/cos(a));
  yp:=y2-round(r*sin(a));
  a:=pi-(4*pi/5);
  xp:=round(x2-r*sin(a));
end;

procedure TjanRPentagonButton.SetBounds (ALeft, ATop,
  AWidth, AHeight: Integer);
var
  hRegion: THandle;
  poly: array[0..4] of Tpoint;
  x2,y2:integer;
begin
  inherited SetBounds (ALeft, ATop, AWidth, AHeight);
  if HandleAllocated then
  begin
  x2:=Awidth div 2;
  y2:=Aheight div 2;
  calcpentagon(Awidth,aheight);
  poly[0]:=point(xp,0);
  poly[1]:=point(Awidth-xp,0);
  poly[2]:=point(Awidth,Aheight-yp);
  poly[3]:=point(x2,Aheight);
  poly[4]:=point(0,Aheight-yp);

  hRegion := CreatePolygonRgn(poly,5,WINDING);
  SetWindowRgn (Handle, hRegion, True);
  end;
end;

procedure TjanRPentagonButton.CNDrawItem(var Msg: TWMDrawItem);
var
  OdsDown, OdsFocus, ActionFocus: Boolean;
  Rect: TRect;
  poly: array[0..5] of Tpoint;
  polyBR:array[0..2] of TPoint;
  polyTL:array[0..3] of TPoint;
  x2,y2,w,h:integer;

  procedure setpoly;
  begin
    w:=Rect.right-Rect.left+1;
    h:=rect.bottom-Rect.top+1;
    x2:=w div 2;
    y2:=h div 2;
    poly[0]:=point(Rect.left+xp,Rect.top);
    poly[1]:=point(rect.right-xp,Rect.top);
    poly[2]:=point(Rect.right,Rect.bottom-yp);
    poly[3]:=point(rect.left+x2,Rect.bottom);
    poly[4]:=point(Rect.Left,Rect.bottom-yp);
    poly[5]:=poly[0];
  end;

begin
  // initialize
  FCanvas.Handle := Msg.DrawItemStruct^.hDC;
  Rect := ClientRect;
  Dec (Rect.Right);
  Dec (Rect.Bottom);
  setpoly;
  with Msg.DrawItemStruct^ do
  begin
    OdsDown := itemState and ODS_SELECTED <> 0;
    OdsFocus := itemState and ODS_FOCUS <> 0;
    ActionFocus := ItemAction = oda_Focus
  end;
  bm.width:=width;
  bm.height:=height;
  with bm.Canvas do
  begin
    pen.width:=2;
    Brush.Color := Color;
    if not ActionFocus then
    begin
      // fill with current color
      Brush.Style := bsSolid;
      FillRect (Rect);
    end;
    // do not fill any more
    Brush.Style := bsClear;
    // draw border if default


{    if Default or OdsFocus then
    begin
      Pen.Color := clWindowFrame;
      if not ActionFocus then
        polyLine(poly);
      // reduce the area for further operations
      InflateRect (Rect, -1, -1);
    end;}
    // test code:
    //InflateRect (Rect, -1, -1);

    if FFlat and (not OdsDown) and (not IsHot) and (not (csdesigning in componentstate)) then
    begin
      Pen.Color := FFlatbordercolor;
      polyLine(poly);
    end
    else if OdsDown then
    begin
      // draw gray border all around
      Pen.Color := clBtnShadow;
      if not ActionFocus then
        polyline(poly);
      // gray border (bottom-right)
      Pen.Color :=  clwhite;
      setpoly;
      polyBR[0]:=poly[1];
      polyBR[1]:=poly[2];
      polyBR[2]:=poly[3];
      polyline(polyBR);
      // white border (top-left)
      Pen.Color :=  clwindowframe;
      polyTL[0]:=poly[3];
      polyTL[1]:=poly[4];
      polyTL[2]:=poly[0];
      polyTL[3]:=poly[1];
      polyline(polyTL);
      // gray border (bottom-right, internal)
      Pen.Color := clBtnShadow;
      InflateRect (Rect, -1, -1);
      setpoly;
      polyBR[0]:=poly[1];
      polyBR[1]:=poly[2];
      polyBR[2]:=poly[3];
      polyline(polyBR);
    end
    else if not ActionFocus then
    begin
      // gray border (bottom-right)
      Pen.Color :=  clWindowFrame;
      setpoly;
      polyBR[0]:=poly[1];
      polyBR[1]:=poly[2];
      polyBR[2]:=poly[3];
      polyline(polyBR);
      // white border (top-left)
      Pen.Color :=  clWhite;
      polyTL[0]:=poly[3];
      polyTL[1]:=poly[4];
      polyTL[2]:=poly[0];
      polyTL[3]:=poly[1];
      polyline(polyTL);
      // gray border (bottom-right, internal)
      Pen.Color := clBtnShadow;
      InflateRect (Rect, -1, -1);
      setpoly;
      polyBR[0]:=poly[1];
      polyBR[1]:=poly[2];
      polyBR[2]:=poly[3];
      polyline(polyBR);
    end;
    // smooth edges
    antialias(bm);
    // draw the caption
    InflateRect (Rect, - Width div 5, - Height div 5);
    if OdsDown then
    begin
      Inc (Rect.Left, 2);
      Inc (Rect.Top, 2);
    end;
    Font := Self.Font;
    if (IsHot and (not OdsDown)) then
      font.color:=FHotColor;
    if not ActionFocus then
      DrawText (bm.Canvas.Handle, PChar (Caption), -1,
        Rect, dt_SingleLine or dt_Center or dt_VCenter);

    // draw the focus rect around the text
    Brush.Style := bsSolid;
    Pen.Color:= clBlack;
    Brush.Color := clWhite;
    if IsFocused or OdsFocus or ActionFocus then
      DrawFocusRect (Rect);
  end; // with bm.Canvas and if DrawEntire
  Fcanvas.Draw (0,0,bm);
  FCanvas.Handle := 0;
  Msg.Result := 1; // message handled
end;

procedure TjanRPentagonButton.CMFontChanged(var Msg: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TjanRPentagonButton.CMEnabledChanged(var Msg: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TjanRPentagonButton.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  Perform(WM_LBUTTONDOWN, Message.Keys, Longint(Message.Pos));
end;

procedure TjanRPentagonButton.SetButtonStyle (ADefault: Boolean);
begin
  if ADefault <> IsFocused then
  begin
    IsFocused := ADefault;
    Invalidate;
  end;
end;


procedure TjanRPentagonButton.CMMouseEnter(var Message: TMessage);
begin
  IsHot:=true;
  invalidate;
end;

procedure TjanRPentagonButton.CMMouseLeave(var Message: TMessage);
begin
  IsHot:=false;
  invalidate;
end;

procedure TjanRPentagonButton.SetHotColor(const Value: TColor);
begin
  FHotColor := Value;
end;

procedure TjanRPentagonButton.SetFlat(const Value: boolean);
begin
  FFlat := Value;
  invalidate;
end;

procedure TjanRPentagonButton.SetFlatBorderColor(const Value: TColor);
begin
  FFlatBorderColor := Value;
end;


end.
