unit janRingButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,janButtonUtils;

type
  TjanRingButton = class(TButton)
  private
    IsFocused: Boolean;
    FCanvas: TCanvas;
    bm:TBitmap;
    procedure CNDrawItem(var Msg: TWMDrawItem); message CN_DRAWITEM;
    procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
    procedure CMEnabledChanged(var Msg: TMessage); message CM_ENABLEDCHANGED;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk);
      message WM_LBUTTONDBLCLK;
  protected
    procedure SetBounds (ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure SetButtonStyle(ADefault: Boolean); override;
    procedure Resize; override;
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Color;
    property Width default 100;
    property Height default 100;
    property ParentShowHint;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnEnter;
    property OnExit;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('janButtons', [TjanRingButton]);
end;


constructor TjanRingButton.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  bm:=Tbitmap.create;
  SetBounds (Left, Top, 100, 100);
  FCanvas := TCanvas.Create;
end;

destructor TjanRingButton.Destroy;
begin
  inherited Destroy;
  FCanvas.Free;
  bm.free;
end;

procedure TjanRingButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params
    do Style := Style or bs_OwnerDraw;
end;

procedure TjanRingButton.CreateWnd;
var
  rgn1,rgn2,rgn3: Hrgn;
  x4,y4:integer;
begin
  inherited CreateWnd;
    x4:=Width div 4;
    y4:=Height div 4;
    rgn3 := CreateEllipticRgn (0, 0, Width, Height);
    rgn1 := CreateEllipticRgn (0, 0, Width, Height);
    rgn2 := CreateEllipticRgn (x4, y4, Width-x4, Height-x4);
    Combinergn(rgn3,rgn1,rgn2,RGN_XOR);
    SetWindowRgn (Handle, rgn3, True);
end;

procedure TjanRingButton.SetBounds (ALeft, ATop,
  AWidth, AHeight: Integer);
var
  rgn1,rgn2,rgn3: Hrgn;
  x4,y4:integer;
begin
  inherited SetBounds (ALeft, ATop, AWidth, AHeight);
  if HandleAllocated then
  begin
    x4:=AWidth div 4;
    y4:=aheight div 4;
    rgn1 := CreateEllipticRgn (0, 0, AWidth, AHeight);
    rgn1 := CreateEllipticRgn (0, 0, AWidth, AHeight);
    rgn2 := CreateEllipticRgn (x4, y4, AWidth-x4, AHeight-x4);
    Combinergn(rgn3,rgn1,rgn2,RGN_XOR);
    SetWindowRgn (Handle, rgn3, True);
  end;
end;

procedure TjanRingButton.CNDrawItem(var Msg: TWMDrawItem);
var
  OdsDown, OdsFocus, ActionFocus: Boolean;
  R,Ri: TRect;
  x4,y4:integer;
begin
  // initialize
  x4:=(width div 4)-1;
  y4:=(height div 4) -1;
  FCanvas.Handle := Msg.DrawItemStruct^.hDC;
  R := ClientRect;
  Ri:=Rect(R.left+x4,R.top+y4,R.right-x4,R.bottom-y4);
  Dec (R.Right);
  Dec (R.Bottom);
  with Msg.DrawItemStruct^ do
  begin
    OdsDown := itemState and ODS_SELECTED <> 0;
    OdsFocus := itemState and ODS_FOCUS <> 0;
    ActionFocus := ItemAction = oda_Focus
  end;

  bm.width:=width;
  bm.height:=height;
  bm.PixelFormat:=pf24bit;

  with bm.Canvas do
  begin
    pen.width:=2;
    Brush.Color := Color;
    if not ActionFocus then
    begin
      // fill with current color
      Brush.Style := bsSolid;
      FillRect (R);
    end;
    // do not fill any more
    Brush.Style := bsClear;
    // draw border if default
    if Default or OdsFocus then
    begin
      Pen.Color := clWindowFrame;
      if not ActionFocus then begin
        Ellipse (R.Left, R.Top, R.Right, R.Bottom);
        Ellipse (Ri.Left, Ri.Top, Ri.Right, Ri.Bottom);
       end;
      // reduce the area for further operations
      InflateRect (R, -1, -1);
      InflateRect(Ri,1,1);
    end;

    if OdsDown then
    begin
      // draw gray border all around
      Pen.Color := clBtnShadow;
      if not ActionFocus then begin
        Ellipse (R.Left, R.Top, R.Right, R.Bottom);
        Ellipse (Ri.Left, Ri.Top, Ri.right, Ri.Bottom);
      end;
      // white border (bottom-right)
      Pen.Color :=  clwhite;
      Arc (R.Left, R.Top, R.Right, R.Bottom, // ellipse
        R.Left, R.Bottom, // start
        R.Right, R.Top); // end
      Pen.Color :=  clbtnshadow;
      Arc (Ri.Left, Ri.Top, Ri.Right, Ri.Bottom, // ellipse
        R.Left, R.Bottom, // start
        R.Right, R.Top); // end

      // gray border (top-left)
      Pen.Color :=  clbtnshadow;
      Arc (R.Left, R.Top, R.Right, R.Bottom, // ellipse
        R.Right, R.Top, // start
        R.Left, R.Bottom); // end
      Pen.Color :=  clbtnhighlight;
      Arc (Ri.Left, Ri.Top, Ri.Right, Ri.Bottom, // ellipse
        R.Right, R.Top, // start
        R.Left, R.Bottom); // end


      // gray border (top-left, internal)
      Pen.Color := clBtnShadow;
      InflateRect (R, -1, -1);
      InflateRect (Ri, 1, 1);
//      Arc (Rect.Left, Rect.Top, Rect.Right, Rect.Bottom, // ellipse
//        Rect.Right, Rect.Top, // start
//        Rect.Left, Rect.Bottom); // end
    end
    else if not ActionFocus then
    begin
      // gray border (bottom-right)
      Pen.Color :=  clWindowFrame;
      Arc (R.Left, R.Top, R.Right, R.Bottom, // ellipse
        R.Left, R.Bottom, // start
        R.Right, R.Top); // end
      Pen.Color :=  clbtnhighlight;
      Arc (Ri.Left, Ri.Top, Ri.Right, Ri.Bottom, // ellipse
        R.Left, R.Bottom, // start
        R.Right, R.Top); // end

      // white border (top-left)
      Pen.Color :=  clWhite;
      Arc (R.Left, R.Top, R.Right, R.Bottom, // ellipse
        R.Right, R.Top, // start
        R.Left, R.Bottom); // end
      Pen.Color :=  clbtnshadow;
      Arc (Ri.Left, Ri.Top, Ri.Right, Ri.Bottom, // ellipse
        R.Right, R.Top, // start
        R.Left, R.Bottom); // end


      // gray border (bottom-right, internal)
      Pen.Color := clBtnShadow;
      InflateRect (R, -1, -1);
      InflateRect (Ri, 1, 1);
      Arc (R.Left, R.Top, R.Right, R.Bottom, // ellipse
        R.Left, R.Bottom, // start
        R.Right, R.Top); // end
      Pen.Color := clBtnHighlight;
      Arc (Ri.Left, Ri.Top, Ri.Right, Ri.Bottom, // ellipse
        R.Left, R.Bottom, // start
        R.Right, R.Top); // end

    end;
    // smooth edges
    antialias(bm);
    // draw the caption
{    InflateRect (Rect, - Width div 5, - Height div 5);
    if OdsDown then
    begin
      Inc (Rect.Left, 2);
      Inc (Rect.Top, 2);
    end;
    Font := Self.Font;
    if not ActionFocus then
      DrawText (bm.Canvas.Handle, PChar (Caption), -1,
        Rect, dt_SingleLine or dt_Center or dt_VCenter);

    // draw the focus rect around the text
    Brush.Style := bsSolid;
    Pen.Color:= clBlack;
    Brush.Color := clWhite;
    if IsFocused or OdsFocus or ActionFocus then
      DrawFocusRect (Rect);}
  end; // with FCanvas and if DrawEntire
  FCanvas.Draw(0,0,bm);
  FCanvas.Handle := 0;
  Msg.Result := 1; // message handled
end;

procedure TjanRingButton.CMFontChanged(var Msg: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TjanRingButton.CMEnabledChanged(var Msg: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TjanRingButton.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  Perform(WM_LBUTTONDOWN, Message.Keys, Longint(Message.Pos));
end;

procedure TjanRingButton.SetButtonStyle (ADefault: Boolean);
begin
  if ADefault <> IsFocused then
  begin
    IsFocused := ADefault;
    Invalidate;
  end;
end;




procedure TjanRingButton.Resize;
var
  rgn1,rgn2,rgn3: Hrgn;
  x4,y4:integer;
begin
    x4:=Width div 4;
    y4:=Height div 4;
    rgn3 := CreateEllipticRgn (0, 0, Width, Height);
    rgn1 := CreateEllipticRgn (0, 0, Width, Height);
    rgn2 := CreateEllipticRgn (x4, y4, Width-x4, Height-x4);
    Combinergn(rgn3,rgn1,rgn2,RGN_XOR);
    SetWindowRgn (Handle, rgn3, True);
end;

end.
